﻿Type=Class
Version=4.2
ModulesStructureVersion=1
B4J=true
@EndOfDesignText@
'Class module
Sub Class_Globals
	Private fx As JFX
	Private apParent As AnchorPane
	Private grd As jGridPane
	
	Private ROWS, COLUMNS As Int
	Private ITERATION As Int = 0
	Private SHOW_DEBUG_COLORS As Boolean = False
	
	Private mapPos As Map
	Private mapActive As Map

	Private Time As Timer
End Sub

'Initializes the object. You can add parameters to this method if needed.
Public Sub Initialize(ap As AnchorPane, iColumns As Int, iRows As Int)
	apParent = ap
	
	grd.Initialize
	grd.GridLinesVisible = True	
	
	mapPos.Initialize
	mapActive.Initialize
	
	apParent.AddNode(grd.GridPane, 0, 0, -1, -1)
	apParent.SetAnchors(grd.GridPane, 0, 0, 0, 0)
	
	Time.Initialize("Time", 100)
	Time.Enabled = False
	
	BuildGrid(iColumns, iRows)	
	
End Sub

Public Sub Start 
	Time.Enabled = True
	grd.GridLinesVisible = False
End Sub

Public Sub Pause 
	Time.Enabled = False
	grd.GridLinesVisible = True
End Sub

Public Sub setDebugEnabled(Value As Boolean)
	SHOW_DEBUG_COLORS = Value
End Sub

Public Sub SetState(Column As Int, Row As Int, State As Int)
	Dim n As Node = grd.GetChildByPosition(Column, Row)
	If n <> Null Then
		Dim t As Map = n.Tag
		t.Put(Abs((ITERATION mod 2) -1), State)
		If State = 1 Then
			n.Style = $"-fx-background-color:black;"$
		Else
			n.Style = $"-fx-background-color:transparent;"$
		End If	
	End If		
End Sub

Private Sub BuildGrid(iColumns As Int, iRows As Int)
	ROWS = iRows
	COLUMNS = iColumns
	
	'Build the columns:		
	For I = 1 To COLUMNS
		grd.Col_setHgrow(grd.Col_newColumn2(0), grd.PRIORITY_NEVER)
	Next
	grd.Col_setWidthAllEqual

	'Build the columns:		
	For I = 1 To ROWS
		grd.Row_setVgrow(grd.Row_newRow2(0), grd.PRIORITY_NEVER)
	Next
	grd.Row_setHeightAllEqual
	
	For Row = 0 To ROWS -1
		For Column = 0 To COLUMNS -1
			Dim ap As AnchorPane
			ap.Initialize("Draw")
			ap.Tag = CreateMap(0:0, 1:0, "Column":Column, "Row":Row)
		
			grd.Add(ap, Column, Row)
			mapPos.Put(Column & Row, ap)
		Next
	Next	
End Sub

Private Sub Draw_MouseClicked (EventData As MouseEvent)
	Dim ap As AnchorPane = Sender
	Dim Data As Map = ap.Tag
	Data.Put(Abs((ITERATION mod 2) -1), 1)
	ap.Style = $"-fx-background-color:black;"$
	mapActive.Put(ap, False)
	Dim Neighbours As Map = GetNeighbours(Abs((ITERATION mod 2) -1), Data.Get("Column"), Data.Get("Row"))
	Dim NeighboursAlive As List = Neighbours.Get("Alive")
	Dim NeighboursDead As List = Neighbours.Get("Dead")	
	For Each Neighbour As AnchorPane In NeighboursAlive
		mapActive.Put(Neighbour, True)
	Next
	For Each Neighbour As AnchorPane In NeighboursDead
		mapActive.Put(Neighbour, True)
	Next	
End Sub

Private Sub Time_Tick
	ITERATION = ITERATION + 1
	
	Dim ThisState As Int = ITERATION mod 2
	Dim NextState As Int = Abs(ThisState -1)

	Dim ActiveNext As Map
	ActiveNext.Initialize
	For Each ap As AnchorPane In mapActive.Keys
		
		Dim Data As Map = ap.Tag
		Dim Neighbours As Map = GetNeighbours(ThisState, Data.Get("Column"), Data.Get("Row"))
		Dim NeighboursAlive As List = Neighbours.Get("Alive")
		Dim NeighboursDead As List = Neighbours.Get("Dead")

		Dim IsActive As Boolean = False
			
		'Get current state:
		Select Data.Get(ThisState) 
			Case 0
				If NeighboursAlive.Size = 3 Then
					Data.Put(NextState, 1)
					ap.Style = $"-fx-background-color:black;"$
					IsActive = True
				Else
					Data.Put(NextState, 0)
				End If
			Case 1
				If NeighboursAlive.Size < 2 Or NeighboursAlive.Size > 3 Then
					Data.Put(NextState, 0)
					ap.Style = $"-fx-background-color:transparent;"$
				Else
					Data.Put(NextState, 1)	
					IsActive = True
				End If
		End Select

		If NeighboursAlive.Size > 0 Then
			IsActive = True
		End If

		If IsActive Then
			If SHOW_DEBUG_COLORS Then ap.Style =  ap.Style & $"-fx-border-color:red;"$	

			ActiveNext.Put(ap, False)

			For Each Neighbour As AnchorPane In NeighboursAlive
				ActiveNext.Put(Neighbour, True)
			Next
			For Each Neighbour As AnchorPane In NeighboursDead
				ActiveNext.Put(Neighbour, True)
			Next
						
		Else
			If SHOW_DEBUG_COLORS Then ap.Style = ap.Style & $"-fx-border-color:transparent;"$	
	
		End If	
	Next
	
	mapActive.Clear
	For i = 0 To ActiveNext.size -1
		mapActive.Put(ActiveNext.GetKeyAt(i), ActiveNext.GetValueAt(i))
	Next	
	
End Sub

Private Sub GetNeighbours(IterationState As Int, Column As Int, Row As Int) As Map
	Dim Alive, Dead As List
	Alive.Initialize
	Dead.Initialize
	For NewRow = Max(Row - 1, 0) To Min(Row + 1, ROWS -1) 
		For NewCol = Max(Column - 1, 0) To Min(Column + 1, COLUMNS -1)
			If (NewCol = Column And NewRow = Row) Then Continue

			Dim ap As AnchorPane = mapPos.GetDefault(NewCol & NewRow, Null)
			If ap <> Null Then
				Dim Data As Map = ap.Tag
				If Data.Get(IterationState) = 1 Then
					Alive.Add(ap)
				Else
					Dead.Add(ap)
				End If		
			End If
		Next
	Next
	Return CreateMap("Alive":Alive, "Dead":Dead)
End Sub

'Private Sub GetAliveNumber(Cycle As Int, Neighbours As List) As Int
'	Dim Alive As Int = 0
'	For Each n As Node In Neighbours
'		Dim Data As Map = n.Tag
'		If Data.Get(Cycle) = 1 Then
'			Alive = Alive + 1
'		End If
'	Next
'	Return Alive
'End Sub

'Private Sub GetNeighbourStates(IterationState As Int, Column As Int, Row As Int) As Map
'	Dim m As Map = CreateMap("Alive":0.0, "Dead":0.0)
''	Log($"REQUEST col ${Column} row ${Row}"$)
'	For NewRow = Max(Row - 1, 0) To Min(Row + 1, ROWS -1) 
'		For NewCol = Max(Column - 1, 0) To Min(Column + 1, COLUMNS -1)
'			If (NewCol = Column And NewRow = Row) Then Continue
'			
''			Log($"NewCol ${NewCol} NewRow ${NewRow}"$)
''			Dim n As Node = grd.GetChildByPosition(NewCol, NewRow)
'			Dim n As Node = mapPos.GetDefault(NewCol & NewRow, Null)
'			If n <> Null Then
'				Dim State As Map = n.Tag
'				Select State.Get(IterationState) 
'					Case 0
'						m.Put("Dead", m.Get("Dead") + 1)
'					Case 1
'						m.Put("Alive", m.Get("Alive") + 1)
'				End Select					
'			End If
'		Next
'	Next
''	Log($"RESULT alive ${m.Get("Alive")} dead ${m.Get("Dead")}"$)
'	Return m
'End Sub
